@@ -3,7 +3,7 @@ package Tk::ColorEntry;
 use strict;
 use warnings;
 use vars qw($VERSION);
-$VERSION = '0.05';
+$VERSION = '0.06';
 use Tk;
 
 use base qw(Tk::Derived Tk::Frame);
@@ -79,6 +79,8 @@ Reference to the variable where the current value is held.
 sub Populate {
 	my ($self,$args) = @_;
 
+	my $pop = delete $args->{'-popcolor'};
+	
 	$self->SUPER::Populate($args);
 	
 	my $entry = $self->Entry(
@@ -95,20 +97,21 @@ sub Populate {
 	);
 	$self->Advertise('Display', $indicator);
 	$self->Advertise('Entry', $entry);
-	my $pop = $self->PopColor(
+	$pop = $self->PopColor(
 		-updatecall => sub {
 			$self->put(shift);
 		},
 		-widget => $self,
-	);
-	$self->Advertise('Pop', $pop);
+	) unless defined $pop;
 
-	$entry->bind('<Button-1>', [$self, 'popFlip']);
+	$entry->bind('<Button-1>', [$self, 'popBlock']);
+	$entry->bind('<ButtonRelease-1>', [$self, 'popFlip']);
 	$entry->bind('<Return>', [$self, 'popFlip']);
 	$entry->bind('<FocusOut>', [$self, 'popDown']);
 	$entry->bind('<Key>', [$self, 'OnKey']);
 	$entry->bind('<Escape>', [$self, 'OnEscape']);
 
+	$self->{POPBLOCK} = 0;
 	my $var = '';
 	$self->ConfigSpecs(
 		-background => ['SELF', 'DESCENDANTS'],
@@ -131,6 +134,7 @@ sub Populate {
 		}, undef, undef, 'sunken'],
 		-justify => [$entry],
 		-popborderwidth => [{-borderwidth => $pop}, undef, undef, 1],
+		-popcolor => ['PASSIVE', undef, undef, $pop],
 		-poprelief => [{-relief => $pop}, undef, undef, 'raised'],
 		-state => [$entry],
 		-variable => [{-textvariable => $entry}, undef, undef, \$var],
@@ -190,31 +194,48 @@ sub OnKey {
 sub popCancel {
 	my $self = shift;
 	delete $self->{'e_save'};
-	$self->Subwidget('Pop')->popCancel;
+	$self->cget('-popcolor')->popCancel;
+}
+
+sub popBlock {
+	my $self = shift;
+	if ($self->cget('-popcolor')->ismapped) {
+		my $color = $self->Subwidget('Entry')->get;
+		$self->Callback('-command', $color) if $self->validate($color);
+		$self->{POPBLOCK} = 1;
+		$self->after(400, sub { $self->{POPBLOCK} = 0});
+	}
 }
 
 sub popDown {
 	my $self = shift;
+	print "popdown\n";
 	delete $self->{'e_save'};
-	$self->Subwidget('Pop')->popDown;
 	my $color = $self->Subwidget('Entry')->get;
 	$self->Callback('-command', $color) if $self->validate($color);
+	$self->cget('-popcolor')->popDown;
 }
 
 sub popFlip {
 	my $self = shift;
-	if ($self->Subwidget('Pop')->ismapped) {
+	if ($self->cget('-popcolor')->ismapped) {
 		$self->popDown
 	} else {
-		$self->popUp
+		$self->popUp unless $self->{POPBLOCK}
 	}
 }
 
 sub popUp {
 	my $self = shift;
+	print "popup\n";
 	my $save = $self->Subwidget('Entry')->get;
 	$self->{'e_save'} = $save;
-	$self->Subwidget('Pop')->popUp;
+	my $pop = $self->cget('-popcolor');
+	$pop->configure(-widget => $self->Subwidget('Entry'));
+	$pop->configure(-updatecall => ['put', $self]);
+	$pop->put($save);
+	
+	$pop->popUp;
 }
 
 =item B<put>(I<$color>)
@@ -232,10 +253,15 @@ sub put {
 	}
 	my $var = $self->Subwidget('Entry')->cget('-textvariable');
 	$$var = $color;
-	$self->Subwidget('Pop')->put($color);
 	$self->EntryUpdate;
 }
 
+sub validate {
+	my ($self, $val) = @_;
+	my $repeat = $self->cget('-popcolor')->colordepth / 4;
+	return $val =~ /^#(?:[0-9a-fA-F]{3}){$repeat}$/
+}
+
 =back
 
 =head1 LICENSE
